@@ -23,8 +23,12 @@
 // support up to version 3 is removed as serialization/deserialization
 // depends on the size of the class.
 //
-
-#include "gu_disable_non_virtual_dtor.hpp"
+#if defined(__GNUG__)
+# if (__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || (__GNUC__ > 4)
+#  pragma GCC diagnostic push
+# endif // (__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || (__GNUC__ > 4)
+# pragma GCC diagnostic ignored "-Weffc++"
+#endif
 
 //
 // Sender                            Receiver
@@ -296,23 +300,26 @@ namespace galera
                 }
             }
 
-            void send_handshake(gu::AsioSocket& socket)
+            template <class ST>
+            void send_handshake(ST& socket)
             {
                 Handshake  hs(version_);
                 gu::Buffer buf(hs.serial_size());
                 size_t offset(hs.serialize(&buf[0], buf.size(), 0));
-                size_t n(socket.write(gu::AsioConstBuffer(&buf[0], buf.size())));
+                size_t n(asio::write(socket, asio::buffer(&buf[0],
+                                                          buf.size())));
                 if (n != offset)
                 {
                     gu_throw_error(EPROTO) << "error sending handshake";
                 }
             }
 
-            void recv_handshake(gu::AsioSocket& socket)
+            template <class ST>
+            void recv_handshake(ST& socket)
             {
                 Message    msg(version_);
                 gu::Buffer buf(msg.serial_size());
-                size_t n(socket.read(gu::AsioMutableBuffer(&buf[0], buf.size())));
+                size_t n(asio::read(socket, asio::buffer(&buf[0], buf.size())));
 
                 if (n != buf.size())
                 {
@@ -352,12 +359,13 @@ namespace galera
                 // TODO: Figure out protocol versions to use
             }
 
-            void send_handshake_response(gu::AsioSocket& socket)
+            template <class ST>
+            void send_handshake_response(ST& socket)
             {
                 HandshakeResponse hsr(version_);
                 gu::Buffer buf(hsr.serial_size());
                 size_t offset(hsr.serialize(&buf[0], buf.size(), 0));
-                size_t n(socket.write(gu::AsioConstBuffer(&buf[0], buf.size())));
+                size_t n(asio::write(socket, asio::buffer(&buf[0], buf.size())));
                 if (n != offset)
                 {
                     gu_throw_error(EPROTO)
@@ -365,11 +373,12 @@ namespace galera
                 }
             }
 
-            void recv_handshake_response(gu::AsioSocket& socket)
+            template <class ST>
+            void recv_handshake_response(ST& socket)
             {
                 Message    msg(version_);
                 gu::Buffer buf(msg.serial_size());
-                size_t n(socket.read(gu::AsioMutableBuffer(&buf[0], buf.size())));
+                size_t n(asio::read(socket, asio::buffer(&buf[0], buf.size())));
 
                 if (n != buf.size())
                 {
@@ -401,23 +410,25 @@ namespace galera
                 }
             }
 
-            void send_ctrl(gu::AsioSocket& socket, int8_t code)
+            template <class ST>
+            void send_ctrl(ST& socket, int8_t code)
             {
                 Ctrl       ctrl(version_, code);
                 gu::Buffer buf(ctrl.serial_size());
                 size_t offset(ctrl.serialize(&buf[0], buf.size(), 0));
-                size_t n(socket.write(gu::AsioConstBuffer(&buf[0], buf.size())));
+                size_t n(asio::write(socket, asio::buffer(&buf[0],buf.size())));
                 if (n != offset)
                 {
                     gu_throw_error(EPROTO) << "error sending ctrl message";
                 }
             }
 
-            int8_t recv_ctrl(gu::AsioSocket& socket)
+            template <class ST>
+            int8_t recv_ctrl(ST& socket)
             {
                 Message    msg(version_);
                 gu::Buffer buf(msg.serial_size());
-                size_t n(socket.read(gu::AsioMutableBuffer(&buf[0], buf.size())));
+                size_t n(asio::read(socket, asio::buffer(&buf[0], buf.size())));
 
                 if (n != buf.size())
                 {
@@ -440,13 +451,16 @@ namespace galera
                 return msg.ctrl();
             }
 
-            void send_ordered(gu::AsioSocket&                           socket,
+
+            template <class ST>
+
+            void send_ordered(ST&                           socket,
                               const gcache::GCache::Buffer& buffer,
                               bool const                    preload_flag)
             {
                 Message::Type type(ordered_type(buffer));
 
-                std::array<gu::AsioConstBuffer, 3> cbs;
+                gu::array<asio::const_buffer, 3>::type cbs;
 
                 size_t      payload_size; /* size of the 2nd cbs buffer */
                 size_t      sent;
@@ -465,8 +479,8 @@ namespace galera
                     {
                         payload_size = buffer.size();
                         const void* const ptr(buffer.ptr());
-                        cbs[1] = gu::AsioConstBuffer(ptr, payload_size);
-                        cbs[2] = gu::AsioConstBuffer(ptr, 0);
+                        cbs[1] = asio::const_buffer(ptr, payload_size);
+                        cbs[2] = asio::const_buffer(ptr, 0);
 
                         if (gu_likely(Message::T_TRX == type)) // compatibility
                         {
@@ -482,8 +496,8 @@ namespace galera
                         WriteSetIn::GatherVector out;
                         payload_size = ws.gather (out, false, false);
                         assert (2 == out->size());
-                        cbs[1] = gu::AsioConstBuffer(out[0].ptr, out[0].size);
-                        cbs[2] = gu::AsioConstBuffer(out[1].ptr, out[1].size);
+                        cbs[1] = asio::const_buffer(out[0].ptr, out[0].size);
+                        cbs[2] = asio::const_buffer(out[1].ptr, out[1].size);
 
                         seqno_d = buffer.seqno_g() - ws.pa_range();
 
@@ -522,34 +536,37 @@ namespace galera
                                             &buf[0], buf.size(), offset);
                 }
 
-                cbs[0] = gu::AsioConstBuffer(&buf[0], buf.size());
+                cbs[0] = asio::const_buffer(&buf[0], buf.size());
 
                 if (gu_likely(payload_size))
                 {
-                    sent = gu::write(socket, cbs);
+                    sent = asio::write(socket, cbs);
                 }
                 else
                 {
-                    sent = socket.write(cbs[0]);
+                    sent = asio::write(socket, asio::buffer(cbs[0]));
                 }
 
                 log_debug << "sent " << sent << " bytes";
             }
 
-            void skip_bytes(gu::AsioSocket& socket, size_t bytes)
+            template <class ST>
+            void skip_bytes(ST& socket, size_t bytes)
             {
                 gu::Buffer buf(4092);
                 while (bytes > 0)
                 {
-                    bytes -= socket.read(
-                        gu::AsioMutableBuffer(
-                            &buf[0], std::min(buf.size(), bytes)));
+                    bytes -= asio::read(
+                        socket,
+                        asio::buffer(&buf[0], std::min(buf.size(), bytes)));
                 }
                 assert(bytes == 0);
             }
 
+
+            template <class ST>
             void
-            recv_ordered(gu::AsioSocket& socket,
+            recv_ordered(ST& socket,
                          std::pair<gcs_action, bool>& ret)
             {
                 gcs_action& act(ret.first);
@@ -562,7 +579,7 @@ namespace galera
 
                 Message    msg(version_);
                 gu::Buffer buf(msg.serial_size());
-                size_t n(socket.read(gu::AsioMutableBuffer(&buf[0], buf.size())));
+                size_t n(asio::read(socket, asio::buffer(&buf[0], buf.size())));
 
                 if (n != buf.size())
                 {
@@ -591,7 +608,7 @@ namespace galera
 
                         buf.resize(sizeof(seqno_g) + sizeof(seqno_d));
 
-                        n = socket.read(gu::AsioMutableBuffer(&buf[0],buf.size()));
+                        n = asio::read(socket, asio::buffer(&buf[0],buf.size()));
                         if (n != buf.size())
                         {
                             assert(0);
@@ -669,14 +686,12 @@ namespace galera
 
                             void*   const ptr(gcache_.malloc(wsize));
                             ssize_t const r
-                                (socket.read(gu::AsioMutableBuffer(ptr, wsize)));
+                                (asio::read(socket, asio::buffer(ptr, wsize)));
 
                             if (gu_unlikely(r != wsize))
                             {
                                 gu_throw_error(EPROTO)
-                                    << "error reading write set data, "
-                                    << "expected " << wsize
-                                    << " bytes, got " << r << " bytes";
+                                    << "error reading write set data";
                             }
 
                             wbuf = ptr;
@@ -724,8 +739,7 @@ namespace galera
                         }
                         else
                         {
-                            gu_throw_error(-msg.ctrl())
-                                << "peer reported error: " << -msg.ctrl();
+                            gu_throw_error(-msg.ctrl()) <<"peer reported error";
                         }
                     }
                 default:
@@ -776,6 +790,11 @@ namespace galera
     }
 }
 
-#include "gu_enable_non_virtual_dtor.hpp"
+#if defined(__GNUG__)
+# if (__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || (__GNUC__ > 4)
+#  pragma GCC diagnostic pop
+# endif // (__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || (__GNUC__ > 4)
+#endif
+
 
 #endif // GALERA_IST_PROTO_HPP
