@@ -5,13 +5,10 @@
 
 #include "../gcs_sm.hpp"
 
-#include <check.h>
 #include <math.h> // fabs
 #include <string.h>
 
-#include "gcs_sm_test.hpp"
-
-
+#include "gcs_sm_test.hpp" // must be included last
 
 #define TEST_USLEEP 10000
 
@@ -280,7 +277,7 @@ START_TEST (gcs_sm_test_pause)
     long long tmp;
     gcs_sm_stats_get (sm, &q_len, &q_len_max, &q_len_min, &q_len_avg,
                       &tmp, &paused_avg);
-    fail_if (tmp <= paused_ns); paused_ns = tmp;
+    fail_if (tmp < paused_ns); paused_ns = tmp;
     fail_if (paused_avg <= 0.0);
     fail_if (fabs(q_len_avg) > EPS,
              "q_len_avg: expected <= %e, got %e", EPS, fabs(q_len_avg));
@@ -469,9 +466,9 @@ START_TEST (gcs_sm_test_interrupt)
 
     TEST_INTERRUPT_THREAD(1, thr1);
 
-    TEST_CREATE_THREAD(&thr2, 1, 2, 2); /* test queueing after interrupted */
+    TEST_CREATE_THREAD(&thr2, 1, 2, 1); /* test queueing after interrupted */
 
-    TEST_CREATE_THREAD(&thr3, 2, 3, 3);
+    TEST_CREATE_THREAD(&thr3, 2, 3, 2);
 
     TEST_INTERRUPT_THREAD(3, thr3); /* test interrupting last waiter */
 
@@ -486,7 +483,7 @@ START_TEST (gcs_sm_test_interrupt)
     TEST_CREATE_THREAD(&thr1, 3, 4, 1);
     TEST_INTERRUPT_THREAD(4, thr1);
 
-    TEST_CREATE_THREAD(&thr1, 0, 1, 2);
+    TEST_CREATE_THREAD(&thr1, 0, 1, 1);
     TEST_INTERRUPT_THREAD(1, thr1);
 
     gcs_sm_continue (sm);
