@@ -5,7 +5,7 @@ set -eu
 # $Id$
 
 # Galera library version
-VERSION="26.4.4"
+VERSION="26.4.5"
 
 get_cores()
 {
@@ -389,7 +389,7 @@ build_sources()
 GALERA_VER=$RELEASE
 
 pushd "$build_base"
-GALERA_REV=$(git log --pretty=oneline | wc -l) || \
+GALERA_REV=$(git log --pretty=format:'%h' -n1.) || \
 GALERA_REV=$(bzr revno --tree -q)              || \
 GALERA_REV=$(svn info >&/dev/null && svnversion | sed s/\:/,/g) || \
 GALERA_REV="7983e"
